/* 
 *  rocks/rs.h
 *
 *  Copyright (C) 2001 Victor Zandy
 *  See COPYING for distribution terms.
 */
#ifndef _RS_H_
#define _RS_H_

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <signal.h>

extern int rserrno;
const char *rserr();

/* Options */
extern int rs_opt_noauth;
extern int rs_opt_nointerop;
extern int rs_opt_nohb;
extern int rs_opt_nolog;
extern int rs_opt_noflight;
extern int rs_opt_noudp;
extern int rs_opt_alarm_period;
extern int rs_opt_max_alarm_misses;
extern int rs_opt_auth_timeout;
extern int rs_opt_rec_max_timeouts;

#define ERSUNUSED       256    /* unused */
#define ERSINIT         257    /* rs socket init failed */
#if 0
#define ERSDUPLICATE    258    /* file descriptor already in use */
#define ERSRECOVERY     259    /* rs socket recovery failed */
#define ERSOFFLINE      260    /* i/o on offline rs socket */
#endif

int rs_fork();
unsigned int rs_alarm(unsigned int);
int rs_socket(int domain, int type, int protocol);
int rs_bind(int sd, const struct sockaddr *iaddr, socklen_t addrlen);
int rs_listen(int sd, int backlog);
int rs_accept(int srv_sd, struct sockaddr *addr, int *addrlen);
int rs_connect(int sd, const struct sockaddr *iaddr, socklen_t addrlen);
int rs_close(int fd);
int rs_shutdown(int sd, int how);
int rs_read(int fd, void *buf, size_t len);
int rs_write(int fd, const void *buf, size_t len);
int rs_fdset(fd_set *);
int rs_getsockname(int s, struct sockaddr *name, socklen_t *namelen);
int rs_getpeername(int s, struct sockaddr *name, socklen_t *namelen);
int rs_recv(int sd, void *buf, size_t len, int flags);
int rs_send(int sd, const void *buf, size_t len, int flags);
int rs_recvfrom(int sd, void *buf, size_t len, int flags,
		struct sockaddr *from, socklen_t *fromlen);
int rs_sendto(int sd, const void *msg, size_t len, int flags,
	      const struct sockaddr *to, socklen_t tolen);
ssize_t rs_readv(int sd, const struct iovec *iov, int iovcnt);
ssize_t rs_writev(int sd, const struct iovec *iov, int iovcnt);
int rs_recvmsg(int sd, struct msghdr *msg, int flags);
int rs_sendmsg(int sd, const struct msghdr *msg, int flags);
int rs_select(int n, fd_set *rs, fd_set *ws, fd_set *es,
	      struct timeval *tv);
int rs_recover_bad_rocks(int n, fd_set *fds);

/* Overloaded syscalls needed for our sanity, not for the sockets API */
typedef void (*sighandler_t)(int);
int rs_sigaction(int sig, const struct sigaction *act, struct sigaction *oldact);
sighandler_t rs_signal(int signum, sighandler_t handler);
int rs_sigprocmask(int how, const sigset_t *set, sigset_t *oldset);
int rs_sigsuspend(const sigset_t *mask);

/* util.c */
int rs_xwrite(int sd, void *buf, size_t len);
int rs_xread(int sd, void *buf, size_t len, unsigned long ms);
int rs_reset_on_close(int sd, int onoff);
int rs_reuseaddr(int sd);
int rs_nodelay(int sd, int onoff);
int rs_settcpbuf(int sd, int type, int size);
int rs_nonblock(int sd, int on);
int rs_waitread(int sd, unsigned long ms);
void rs_logbytes(char *bytes, int len);

void rs_init();
int rs_init_sys();
int rs_init_signal();

int rs_setsockopt(int s, int level, int optname,
		  const void *optval, socklen_t optlen);
int rs_fcntl(int fd, int cmd, long arg);
int rs_ioctl(int fd, int cmd, long arg);
int rs_setitimer(int which, const struct itimerval *value,
		 struct itimerval *ovalue);

#endif /* _RS_H_ */
